from http import HTTPStatus
from typing import Dict, Optional

import httpx
from typing_extensions import Any

from ... import errors
from ...models.fine_tuning_request import FineTuningRequest

# from ...client import AuthenticatedClient, Client
from ...types import Response


def _get_kwargs(
    id: int,
) -> Dict[str, Any]:
    _kwargs: Dict[str, Any] = {
        "method": "get",
        "url": f"/projects/gym/admin/finetuningrequests/{id}/",
    }

    return _kwargs


def _parse_response(*, client, response: httpx.Response) -> Optional[FineTuningRequest]:
    if response.status_code == HTTPStatus.OK:
        response_200 = FineTuningRequest.from_dict(response.json())

        return response_200
    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(*, client, response: httpx.Response) -> Response[FineTuningRequest]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def projects_gym_admin_finetuningrequests_retrieve_wrapper(client):
    def projects_gym_admin_finetuningrequests_retrieve_wrapped(
        id: int,
    ) -> FineTuningRequest:
        """Retrieve a specific finetuning_request by its id

        Args:
            id (int):

        Raises:
            errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
            httpx.TimeoutException: If the request takes longer than Client.timeout.

        Returns:
            Response[FineTuningRequest]
        """

        kwargs = _get_kwargs(
            id=id,
        )

        httpx_client = client.get_httpx_client()

        response = httpx_client.request(
            **kwargs,
        )

        return _build_response(client=client, response=response).parsed

    return projects_gym_admin_finetuningrequests_retrieve_wrapped



from typing import Dict, List, Type, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..types import UNSET, Unset

T = TypeVar("T", bound="FineTuningMessageOutputGymAdmin")


class FineTuningMessageOutputGymAdminDict(TypedDict):
    id: int
    fine_tuning_request: int
    title: str
    message: str
    user_prompt: NotRequired[Union[None, Unset, str]]
    pass


@_attrs_define
class FineTuningMessageOutputGymAdmin:
    """
    Attributes:
        id (int):
        fine_tuning_request (int):
        title (str):
        message (str):
        user_prompt (Union[None, Unset, str]):
    """

    id: int
    fine_tuning_request: int
    title: str
    message: str
    user_prompt: Union[None, Unset, str] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        fine_tuning_request = self.fine_tuning_request

        title = self.title

        message = self.message

        user_prompt: Union[None, Unset, str]
        if isinstance(self.user_prompt, Unset):
            user_prompt = UNSET
        else:
            user_prompt = self.user_prompt

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "fine_tuning_request": fine_tuning_request,
                "title": title,
                "message": message,
            }
        )
        if user_prompt is not UNSET:
            field_dict["user_prompt"] = user_prompt

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        id = d.pop("id")

        fine_tuning_request = d.pop("fine_tuning_request")

        title = d.pop("title")

        message = d.pop("message")

        def _parse_user_prompt(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        user_prompt = _parse_user_prompt(d.pop("user_prompt", UNSET))

        fine_tuning_message_output_gym_admin = cls(
            id=id,
            fine_tuning_request=fine_tuning_request,
            title=title,
            message=message,
            user_prompt=user_prompt,
        )

        fine_tuning_message_output_gym_admin.additional_properties = d
        return fine_tuning_message_output_gym_admin

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import json
from typing import Dict, List, Tuple, Type, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, TypedDict, TypeVar

from ..models.encoding_format_enum import EncodingFormatEnum
from ..types import UNSET, Unset

T = TypeVar("T", bound="EmbeddingsInput")


class EmbeddingsInputDict(TypedDict):
    project_id: int
    model: str
    input_: Union[List[List[int]], List[int], List[str], str]
    encoding_format: Union[Unset, EncodingFormatEnum] = EncodingFormatEnum.FLOAT
    pass


@_attrs_define
class EmbeddingsInput:
    """
    Attributes:
        project_id (int): The ID of the project to use.
        model (str): The model to generate the embeddings.
        input_ (Union[List[List[int]], List[int], List[str], str]): Embedding Input
        encoding_format (Union[Unset, EncodingFormatEnum]): * `float` - float
            * `base64` - base64 Default: EncodingFormatEnum.FLOAT.
    """

    project_id: int
    model: str
    input_: Union[List[List[int]], List[int], List[str], str]
    encoding_format: Union[Unset, EncodingFormatEnum] = EncodingFormatEnum.FLOAT

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        project_id = self.project_id

        model = self.model

        input_: Union[List[List[int]], List[int], List[str], str]
        if isinstance(self.input_, list):
            input_ = self.input_

        elif isinstance(self.input_, list):
            input_ = self.input_

        elif isinstance(self.input_, list):
            input_ = []
            for input_type_3_item_data in self.input_:
                input_type_3_item = input_type_3_item_data

                input_.append(input_type_3_item)

        else:
            input_ = self.input_

        encoding_format: Union[Unset, str] = UNSET
        if not isinstance(self.encoding_format, Unset):
            encoding_format = self.encoding_format.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "project_id": project_id,
                "model": model,
                "input": input_,
            }
        )
        if encoding_format is not UNSET:
            field_dict["encoding_format"] = encoding_format

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        project_id = (
            self.project_id
            if isinstance(self.project_id, Unset)
            else (None, str(self.project_id).encode(), "text/plain")
        )

        model = self.model if isinstance(self.model, Unset) else (None, str(self.model).encode(), "text/plain")

        input_: Union[Tuple[None, bytes, str], str]
        if isinstance(self.input_, list):
            _temp_input_ = self.input_
            input_ = (None, json.dumps(_temp_input_).encode(), "application/json")

        elif isinstance(self.input_, list):
            _temp_input_ = self.input_
            input_ = (None, json.dumps(_temp_input_).encode(), "application/json")

        elif isinstance(self.input_, list):
            _temp_input_ = []
            for input_type_3_item_data in self.input_:
                input_type_3_item = input_type_3_item_data

                _temp_input_.append(input_type_3_item)
            input_ = (None, json.dumps(_temp_input_).encode(), "application/json")

        else:
            input_ = self.input_

        encoding_format: Union[Unset, Tuple[None, bytes, str]] = UNSET
        if not isinstance(self.encoding_format, Unset):
            encoding_format = (None, str(self.encoding_format.value).encode(), "text/plain")

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {key: (None, str(value).encode(), "text/plain") for key, value in self.additional_properties.items()}
        )
        field_dict.update(
            {
                "project_id": project_id,
                "model": model,
                "input": input_,
            }
        )
        if encoding_format is not UNSET:
            field_dict["encoding_format"] = encoding_format

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        project_id = d.pop("project_id")

        model = d.pop("model")

        def _parse_input_(data: object) -> Union[List[List[int]], List[int], List[str], str]:
            try:
                if not isinstance(data, list):
                    raise TypeError()
                input_type_1 = cast(List[str], data)

                return input_type_1
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                input_type_2 = cast(List[int], data)

                return input_type_2
            except:  # noqa: E722
                pass
            try:
                if not isinstance(data, list):
                    raise TypeError()
                input_type_3 = []
                _input_type_3 = data
                for input_type_3_item_data in _input_type_3:
                    input_type_3_item = cast(List[int], input_type_3_item_data)

                    input_type_3.append(input_type_3_item)

                return input_type_3
            except:  # noqa: E722
                pass
            return cast(Union[List[List[int]], List[int], List[str], str], data)

        input_ = _parse_input_(d.pop("input"))

        _encoding_format = d.pop("encoding_format", UNSET)
        encoding_format: Union[Unset, EncodingFormatEnum]
        if isinstance(_encoding_format, Unset):
            encoding_format = UNSET
        else:
            encoding_format = EncodingFormatEnum(_encoding_format)

        embeddings_input = cls(
            project_id=project_id,
            model=model,
            input_=input_,
            encoding_format=encoding_format,
        )

        embeddings_input.additional_properties = d
        return embeddings_input

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, List, Type, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..types import UNSET, Unset

T = TypeVar("T", bound="FineTunedModelPromotion")


class FineTunedModelPromotionDict(TypedDict):
    model: int
    alias: NotRequired[Union[None, Unset, str]]
    pass


@_attrs_define
class FineTunedModelPromotion:
    """
    Attributes:
        model (int): Model ID to be promoted
        alias (Union[None, Unset, str]): Alias for the finetuned model
    """

    model: int
    alias: Union[None, Unset, str] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        model = self.model

        alias: Union[None, Unset, str]
        if isinstance(self.alias, Unset):
            alias = UNSET
        else:
            alias = self.alias

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "model": model,
            }
        )
        if alias is not UNSET:
            field_dict["alias"] = alias

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        model = self.model if isinstance(self.model, Unset) else (None, str(self.model).encode(), "text/plain")

        alias: Union[None, Unset, str]
        if isinstance(self.alias, Unset):
            alias = UNSET
        else:
            alias = self.alias

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {key: (None, str(value).encode(), "text/plain") for key, value in self.additional_properties.items()}
        )
        field_dict.update(
            {
                "model": model,
            }
        )
        if alias is not UNSET:
            field_dict["alias"] = alias

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        model = d.pop("model")

        def _parse_alias(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        alias = _parse_alias(d.pop("alias", UNSET))

        fine_tuned_model_promotion = cls(
            model=model,
            alias=alias,
        )

        fine_tuned_model_promotion.additional_properties = d
        return fine_tuned_model_promotion

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
import json
from typing import Dict, List, Tuple, Type, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..models.fine_tuning_job_hyperparameters import FineTuningJobHyperparameters
from ..types import UNSET, Unset

T = TypeVar("T", bound="FineTuningJobCreateGymAdmin")


class FineTuningJobCreateGymAdminDict(TypedDict):
    id: int
    fine_tuning_request: int
    created_at: datetime.datetime
    updated_at: datetime.datetime
    training_dataset: NotRequired[Union[Unset, int]]
    validation_dataset: NotRequired[Union[Unset, int]]
    hyperparameters: NotRequired[Union[Unset, FineTuningJobHyperparameters]]
    pass


@_attrs_define
class FineTuningJobCreateGymAdmin:
    """
    Attributes:
        id (int):
        fine_tuning_request (int): Fine-tuning request id to be used for fine-tuning
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        training_dataset (Union[Unset, int]): Training dataset id to be used for fine-tuning. If omitted, the dataset
            from the finetuning_request will be used.
        validation_dataset (Union[Unset, int]): Validation dataset id to be used for fine-tuning
        hyperparameters (Union[Unset, FineTuningJobHyperparameters]):
    """

    id: int
    fine_tuning_request: int
    created_at: datetime.datetime
    updated_at: datetime.datetime
    training_dataset: Union[Unset, int] = UNSET
    validation_dataset: Union[Unset, int] = UNSET
    hyperparameters: Union[Unset, "FineTuningJobHyperparameters"] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        fine_tuning_request = self.fine_tuning_request

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        training_dataset = self.training_dataset

        validation_dataset = self.validation_dataset

        hyperparameters: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.hyperparameters, Unset):
            hyperparameters = self.hyperparameters.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "fine_tuning_request": fine_tuning_request,
                "created_at": created_at,
                "updated_at": updated_at,
            }
        )
        if training_dataset is not UNSET:
            field_dict["training_dataset"] = training_dataset
        if validation_dataset is not UNSET:
            field_dict["validation_dataset"] = validation_dataset
        if hyperparameters is not UNSET:
            field_dict["hyperparameters"] = hyperparameters

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        id = self.id if isinstance(self.id, Unset) else (None, str(self.id).encode(), "text/plain")

        fine_tuning_request = (
            self.fine_tuning_request
            if isinstance(self.fine_tuning_request, Unset)
            else (None, str(self.fine_tuning_request).encode(), "text/plain")
        )

        created_at = self.created_at.isoformat().encode()

        updated_at = self.updated_at.isoformat().encode()

        training_dataset = (
            self.training_dataset
            if isinstance(self.training_dataset, Unset)
            else (None, str(self.training_dataset).encode(), "text/plain")
        )

        validation_dataset = (
            self.validation_dataset
            if isinstance(self.validation_dataset, Unset)
            else (None, str(self.validation_dataset).encode(), "text/plain")
        )

        hyperparameters: Union[Unset, Tuple[None, bytes, str]] = UNSET
        if not isinstance(self.hyperparameters, Unset):
            hyperparameters = (None, json.dumps(self.hyperparameters.to_dict()).encode(), "application/json")

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {key: (None, str(value).encode(), "text/plain") for key, value in self.additional_properties.items()}
        )
        field_dict.update(
            {
                "id": id,
                "fine_tuning_request": fine_tuning_request,
                "created_at": created_at,
                "updated_at": updated_at,
            }
        )
        if training_dataset is not UNSET:
            field_dict["training_dataset"] = training_dataset
        if validation_dataset is not UNSET:
            field_dict["validation_dataset"] = validation_dataset
        if hyperparameters is not UNSET:
            field_dict["hyperparameters"] = hyperparameters

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.fine_tuning_job_hyperparameters import FineTuningJobHyperparameters

        d = src_dict.copy() if src_dict else {}
        id = d.pop("id")

        fine_tuning_request = d.pop("fine_tuning_request")

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        training_dataset = d.pop("training_dataset", UNSET)

        validation_dataset = d.pop("validation_dataset", UNSET)

        _hyperparameters = d.pop("hyperparameters", UNSET)
        hyperparameters: Union[Unset, FineTuningJobHyperparameters]
        if isinstance(_hyperparameters, Unset):
            hyperparameters = UNSET
        else:
            hyperparameters = FineTuningJobHyperparameters.from_dict(_hyperparameters)

        fine_tuning_job_create_gym_admin = cls(
            id=id,
            fine_tuning_request=fine_tuning_request,
            created_at=created_at,
            updated_at=updated_at,
            training_dataset=training_dataset,
            validation_dataset=validation_dataset,
            hyperparameters=hyperparameters,
        )

        fine_tuning_job_create_gym_admin.additional_properties = d
        return fine_tuning_job_create_gym_admin

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class StatusC4AEnum(str, Enum):
    CANCELLED = "cancelled"
    FAILED = "failed"
    QUEUED = "queued"
    RUNNING = "running"
    SUCCEEDED = "succeeded"
    UNKNOWN = "unknown"

    def __str__(self) -> str:
        return str(self.value)



from typing import Dict, List, Type, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..types import UNSET, Unset

T = TypeVar("T", bound="RetrieveFineTuningResponse")


class RetrieveFineTuningResponseDict(TypedDict):
    id: str
    fine_tuned_model: str
    created_at: int
    status: str
    provider_name: str
    provider_id: str
    status_code: int
    finished_at: NotRequired[Union[Unset, int]]
    error: NotRequired[Union[Unset, str]]
    pass


@_attrs_define
class RetrieveFineTuningResponse:
    """
    Attributes:
        id (str): The ID of the fine-tuning job.
        fine_tuned_model (str): The ID of the fine-tuned model.
        created_at (int): The Unix timestamp (in seconds) of when the fine-tuning job was created.
        status (str): The status of the fine-tuning job.
        provider_name (str): The name of the provider that generated the completion.
        provider_id (str): The ID of the provider that generated the completion.
        status_code (int): The status code of the fine-tuning job.
        finished_at (Union[Unset, int]): The Unix timestamp (in seconds) of when the fine-tuning job was finished.
        error (Union[Unset, str]): The error message of the fine-tuning job.
    """

    id: str
    fine_tuned_model: str
    created_at: int
    status: str
    provider_name: str
    provider_id: str
    status_code: int
    finished_at: Union[Unset, int] = UNSET
    error: Union[Unset, str] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        fine_tuned_model = self.fine_tuned_model

        created_at = self.created_at

        status = self.status

        provider_name = self.provider_name

        provider_id = self.provider_id

        status_code = self.status_code

        finished_at = self.finished_at

        error = self.error

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "fine_tuned_model": fine_tuned_model,
                "created_at": created_at,
                "status": status,
                "provider_name": provider_name,
                "provider_id": provider_id,
                "status_code": status_code,
            }
        )
        if finished_at is not UNSET:
            field_dict["finished_at"] = finished_at
        if error is not UNSET:
            field_dict["error"] = error

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        id = d.pop("id")

        fine_tuned_model = d.pop("fine_tuned_model")

        created_at = d.pop("created_at")

        status = d.pop("status")

        provider_name = d.pop("provider_name")

        provider_id = d.pop("provider_id")

        status_code = d.pop("status_code")

        finished_at = d.pop("finished_at", UNSET)

        error = d.pop("error", UNSET)

        retrieve_fine_tuning_response = cls(
            id=id,
            fine_tuned_model=fine_tuned_model,
            created_at=created_at,
            status=status,
            provider_name=provider_name,
            provider_id=provider_id,
            status_code=status_code,
            finished_at=finished_at,
            error=error,
        )

        retrieve_fine_tuning_response.additional_properties = d
        return retrieve_fine_tuning_response

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ProviderAPIConnectionErrorCodeEnum(str, Enum):
    PROVIDERAPICONNECTIONERROR = "ProviderAPIConnectionError"

    def __str__(self) -> str:
        return str(self.value)



from typing import Dict, List, Type, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..models.fine_tuning_request_fine_tuning_message import FineTuningRequestFineTuningMessage
from ..models.fine_tuning_request_model import FineTuningRequestModel
from ..models.fine_tuning_request_project import FineTuningRequestProject
from ..models.status_d09_enum import StatusD09Enum
from ..types import UNSET, Unset

T = TypeVar("T", bound="FineTuningRequest")


class FineTuningRequestDict(TypedDict):
    id: int
    project: "FineTuningRequestProject"
    model: "FineTuningRequestModel"
    dataset: int
    messages: List["FineTuningRequestFineTuningMessage"]
    current_message: "FineTuningRequestFineTuningMessage"
    status: NotRequired[Union[Unset, StatusD09Enum]]
    current_fine_tuning_job: NotRequired[Union[None, Unset, int]]
    pass


@_attrs_define
class FineTuningRequest:
    """
    Attributes:
        id (int):
        project (FineTuningRequestProject):
        model (FineTuningRequestModel):
        dataset (int):
        messages (List['FineTuningRequestFineTuningMessage']):
        current_message (FineTuningRequestFineTuningMessage):
        status (Union[Unset, StatusD09Enum]): * `queued` - Queued
            * `finetuning` - Fine tuning
            * `done` - Done
            * `failed` - Failed
        current_fine_tuning_job (Union[None, Unset, int]):
    """

    id: int
    project: "FineTuningRequestProject"
    model: "FineTuningRequestModel"
    dataset: int
    messages: List["FineTuningRequestFineTuningMessage"]
    current_message: "FineTuningRequestFineTuningMessage"
    status: Union[Unset, StatusD09Enum] = UNSET
    current_fine_tuning_job: Union[None, Unset, int] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        project = self.project.to_dict()

        model = self.model.to_dict()

        dataset = self.dataset

        messages = []
        for messages_item_data in self.messages:
            messages_item = messages_item_data.to_dict()
            messages.append(messages_item)

        current_message = self.current_message.to_dict()

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        current_fine_tuning_job: Union[None, Unset, int]
        if isinstance(self.current_fine_tuning_job, Unset):
            current_fine_tuning_job = UNSET
        else:
            current_fine_tuning_job = self.current_fine_tuning_job

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "project": project,
                "model": model,
                "dataset": dataset,
                "messages": messages,
                "current_message": current_message,
            }
        )
        if status is not UNSET:
            field_dict["status"] = status
        if current_fine_tuning_job is not UNSET:
            field_dict["current_fine_tuning_job"] = current_fine_tuning_job

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.fine_tuning_request_fine_tuning_message import FineTuningRequestFineTuningMessage
        from ..models.fine_tuning_request_model import FineTuningRequestModel
        from ..models.fine_tuning_request_project import FineTuningRequestProject

        d = src_dict.copy() if src_dict else {}
        id = d.pop("id")

        project = FineTuningRequestProject.from_dict(d.pop("project"))

        model = FineTuningRequestModel.from_dict(d.pop("model"))

        dataset = d.pop("dataset")

        messages = []
        _messages = d.pop("messages")
        for messages_item_data in _messages:
            messages_item = FineTuningRequestFineTuningMessage.from_dict(messages_item_data)

            messages.append(messages_item)

        current_message = FineTuningRequestFineTuningMessage.from_dict(d.pop("current_message"))

        _status = d.pop("status", UNSET)
        status: Union[Unset, StatusD09Enum]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = StatusD09Enum(_status)

        def _parse_current_fine_tuning_job(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        current_fine_tuning_job = _parse_current_fine_tuning_job(d.pop("current_fine_tuning_job", UNSET))

        fine_tuning_request = cls(
            id=id,
            project=project,
            model=model,
            dataset=dataset,
            messages=messages,
            current_message=current_message,
            status=status,
            current_fine_tuning_job=current_fine_tuning_job,
        )

        fine_tuning_request.additional_properties = d
        return fine_tuning_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, List, Type, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..types import UNSET, Unset

T = TypeVar("T", bound="FineTuningRequestProject")


class FineTuningRequestProjectDict(TypedDict):
    id: int
    name: str
    description: NotRequired[Union[None, Unset, str]]
    pass


@_attrs_define
class FineTuningRequestProject:
    """
    Attributes:
        id (int):
        name (str):
        description (Union[None, Unset, str]):
    """

    id: int
    name: str
    description: Union[None, Unset, str] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        name = self.name

        description: Union[None, Unset, str]
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "name": name,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        id = d.pop("id")

        name = d.pop("name")

        def _parse_description(data: object) -> Union[None, Unset, str]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, str], data)

        description = _parse_description(d.pop("description", UNSET))

        fine_tuning_request_project = cls(
            id=id,
            name=name,
            description=description,
        )

        fine_tuning_request_project.additional_properties = d
        return fine_tuning_request_project

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Dict, List, Type, Union, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..models.status_c4a_enum import StatusC4AEnum
from ..types import UNSET, Unset

T = TypeVar("T", bound="FineTuningJobOutputGymAdmin")


class FineTuningJobOutputGymAdminDict(TypedDict):
    id: int
    job_id: str
    finished_at: NotRequired[Union[None, Unset, datetime.datetime]]
    status: NotRequired[Union[Unset, StatusC4AEnum]]
    fine_tuned_model: NotRequired[Union[None, Unset, int]]
    fine_tuning_request: NotRequired[Union[None, Unset, int]]
    training_dataset: NotRequired[Union[None, Unset, int]]
    validation_dataset: NotRequired[Union[None, Unset, int]]
    pass


@_attrs_define
class FineTuningJobOutputGymAdmin:
    """
    Attributes:
        id (int):
        job_id (str):
        finished_at (Union[None, Unset, datetime.datetime]):
        status (Union[Unset, StatusC4AEnum]): * `unknown` - Unknown
            * `queued` - Queued
            * `running` - Running
            * `succeeded` - Succeeded
            * `failed` - Failed
            * `cancelled` - Cancelled
        fine_tuned_model (Union[None, Unset, int]):
        fine_tuning_request (Union[None, Unset, int]):
        training_dataset (Union[None, Unset, int]):
        validation_dataset (Union[None, Unset, int]):
    """

    id: int
    job_id: str
    finished_at: Union[None, Unset, datetime.datetime] = UNSET
    status: Union[Unset, StatusC4AEnum] = UNSET
    fine_tuned_model: Union[None, Unset, int] = UNSET
    fine_tuning_request: Union[None, Unset, int] = UNSET
    training_dataset: Union[None, Unset, int] = UNSET
    validation_dataset: Union[None, Unset, int] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        job_id = self.job_id

        finished_at: Union[None, Unset, str]
        if isinstance(self.finished_at, Unset):
            finished_at = UNSET
        elif isinstance(self.finished_at, datetime.datetime):
            finished_at = self.finished_at.isoformat()
        else:
            finished_at = self.finished_at

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        fine_tuned_model: Union[None, Unset, int]
        if isinstance(self.fine_tuned_model, Unset):
            fine_tuned_model = UNSET
        else:
            fine_tuned_model = self.fine_tuned_model

        fine_tuning_request: Union[None, Unset, int]
        if isinstance(self.fine_tuning_request, Unset):
            fine_tuning_request = UNSET
        else:
            fine_tuning_request = self.fine_tuning_request

        training_dataset: Union[None, Unset, int]
        if isinstance(self.training_dataset, Unset):
            training_dataset = UNSET
        else:
            training_dataset = self.training_dataset

        validation_dataset: Union[None, Unset, int]
        if isinstance(self.validation_dataset, Unset):
            validation_dataset = UNSET
        else:
            validation_dataset = self.validation_dataset

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "job_id": job_id,
            }
        )
        if finished_at is not UNSET:
            field_dict["finished_at"] = finished_at
        if status is not UNSET:
            field_dict["status"] = status
        if fine_tuned_model is not UNSET:
            field_dict["fine_tuned_model"] = fine_tuned_model
        if fine_tuning_request is not UNSET:
            field_dict["fine_tuning_request"] = fine_tuning_request
        if training_dataset is not UNSET:
            field_dict["training_dataset"] = training_dataset
        if validation_dataset is not UNSET:
            field_dict["validation_dataset"] = validation_dataset

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        id = d.pop("id")

        job_id = d.pop("job_id")

        def _parse_finished_at(data: object) -> Union[None, Unset, datetime.datetime]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                finished_at_type_0 = isoparse(data)

                return finished_at_type_0
            except:  # noqa: E722
                pass
            return cast(Union[None, Unset, datetime.datetime], data)

        finished_at = _parse_finished_at(d.pop("finished_at", UNSET))

        _status = d.pop("status", UNSET)
        status: Union[Unset, StatusC4AEnum]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = StatusC4AEnum(_status)

        def _parse_fine_tuned_model(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        fine_tuned_model = _parse_fine_tuned_model(d.pop("fine_tuned_model", UNSET))

        def _parse_fine_tuning_request(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        fine_tuning_request = _parse_fine_tuning_request(d.pop("fine_tuning_request", UNSET))

        def _parse_training_dataset(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        training_dataset = _parse_training_dataset(d.pop("training_dataset", UNSET))

        def _parse_validation_dataset(data: object) -> Union[None, Unset, int]:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(Union[None, Unset, int], data)

        validation_dataset = _parse_validation_dataset(d.pop("validation_dataset", UNSET))

        fine_tuning_job_output_gym_admin = cls(
            id=id,
            job_id=job_id,
            finished_at=finished_at,
            status=status,
            fine_tuned_model=fine_tuned_model,
            fine_tuning_request=fine_tuning_request,
            training_dataset=training_dataset,
            validation_dataset=validation_dataset,
        )

        fine_tuning_job_output_gym_admin.additional_properties = d
        return fine_tuning_job_output_gym_admin

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, List, Type, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, TypedDict, TypeVar

from ..types import UNSET, Unset

T = TypeVar("T", bound="FineTuningJobHyperparameters")


class FineTuningJobHyperparametersDict(TypedDict):
    num_epochs: Union[Unset, int] = 3
    pass


@_attrs_define
class FineTuningJobHyperparameters:
    """
    Attributes:
        num_epochs (Union[Unset, int]): Number of epochs for fine-tuning Default: 3.
    """

    num_epochs: Union[Unset, int] = 3

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        num_epochs = self.num_epochs

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if num_epochs is not UNSET:
            field_dict["num_epochs"] = num_epochs

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        num_epochs = d.pop("num_epochs", UNSET)

        fine_tuning_job_hyperparameters = cls(
            num_epochs=num_epochs,
        )

        fine_tuning_job_hyperparameters.additional_properties = d
        return fine_tuning_job_hyperparameters

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, List, Type

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, TypedDict, TypeVar

from ..types import Unset

T = TypeVar("T", bound="FineTuningMessageCreateGymAdmin")


class FineTuningMessageCreateGymAdminDict(TypedDict):
    id: int
    fine_tuning_request: int
    title: str
    message: str
    pass


@_attrs_define
class FineTuningMessageCreateGymAdmin:
    """
    Attributes:
        id (int):
        fine_tuning_request (int):
        title (str):
        message (str):
    """

    id: int
    fine_tuning_request: int
    title: str
    message: str

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        fine_tuning_request = self.fine_tuning_request

        title = self.title

        message = self.message

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "fine_tuning_request": fine_tuning_request,
                "title": title,
                "message": message,
            }
        )

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        id = self.id if isinstance(self.id, Unset) else (None, str(self.id).encode(), "text/plain")

        fine_tuning_request = (
            self.fine_tuning_request
            if isinstance(self.fine_tuning_request, Unset)
            else (None, str(self.fine_tuning_request).encode(), "text/plain")
        )

        title = self.title if isinstance(self.title, Unset) else (None, str(self.title).encode(), "text/plain")

        message = self.message if isinstance(self.message, Unset) else (None, str(self.message).encode(), "text/plain")

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {key: (None, str(value).encode(), "text/plain") for key, value in self.additional_properties.items()}
        )
        field_dict.update(
            {
                "id": id,
                "fine_tuning_request": fine_tuning_request,
                "title": title,
                "message": message,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        id = d.pop("id")

        fine_tuning_request = d.pop("fine_tuning_request")

        title = d.pop("title")

        message = d.pop("message")

        fine_tuning_message_create_gym_admin = cls(
            id=id,
            fine_tuning_request=fine_tuning_request,
            title=title,
            message=message,
        )

        fine_tuning_message_create_gym_admin.additional_properties = d
        return fine_tuning_message_create_gym_admin

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, List, Type

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, TypedDict, TypeVar

from ..models.provider_api_status_error_code_enum import ProviderAPIStatusErrorCodeEnum

T = TypeVar("T", bound="ProviderAPIStatusError")


class ProviderAPIStatusErrorDict(TypedDict):
    message: str
    code: ProviderAPIStatusErrorCodeEnum
    pass


@_attrs_define
class ProviderAPIStatusError:
    """
    Attributes:
        message (str):
        code (ProviderAPIStatusErrorCodeEnum): * `ProviderAPIStatusError` - ProviderAPIStatusError
    """

    message: str
    code: ProviderAPIStatusErrorCodeEnum

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        message = self.message

        code = self.code.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "message": message,
                "code": code,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        message = d.pop("message")

        code = ProviderAPIStatusErrorCodeEnum(d.pop("code"))

        provider_api_status_error = cls(
            message=message,
            code=code,
        )

        provider_api_status_error.additional_properties = d
        return provider_api_status_error

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, List, Type

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, TypedDict, TypeVar

from ..models.document_type_enum import DocumentTypeEnum
from ..types import Unset

T = TypeVar("T", bound="DocumentInput")


class DocumentInputDict(TypedDict):
    name: str
    content: str
    document_type: DocumentTypeEnum
    pass


@_attrs_define
class DocumentInput:
    """
    Attributes:
        name (str):
        content (str):
        document_type (DocumentTypeEnum): * `pdf` - PDF
            * `docx` - Word
            * `txt` - Text
    """

    name: str
    content: str
    document_type: DocumentTypeEnum

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name

        content = self.content

        document_type = self.document_type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "content": content,
                "document_type": document_type,
            }
        )

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        name = self.name if isinstance(self.name, Unset) else (None, str(self.name).encode(), "text/plain")

        content = self.content if isinstance(self.content, Unset) else (None, str(self.content).encode(), "text/plain")

        document_type = (None, str(self.document_type.value).encode(), "text/plain")

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {key: (None, str(value).encode(), "text/plain") for key, value in self.additional_properties.items()}
        )
        field_dict.update(
            {
                "name": name,
                "content": content,
                "document_type": document_type,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        name = d.pop("name")

        content = d.pop("content")

        document_type = DocumentTypeEnum(d.pop("document_type"))

        document_input = cls(
            name=name,
            content=content,
            document_type=document_type,
        )

        document_input.additional_properties = d
        return document_input

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import json
from typing import Dict, List, Tuple, Type, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..models.fine_tuning_sample import FineTuningSample
from ..types import UNSET, Unset

T = TypeVar("T", bound="FineTuningInput")


class FineTuningInputDict(TypedDict):
    project_id: int
    training_data: List["FineTuningSample"]
    model: NotRequired[Union[Unset, str]]
    validaton_data: NotRequired[Union[Unset, List["FineTuningSample"]]]
    num_epochs: Union[Unset, int] = 1
    pass


@_attrs_define
class FineTuningInput:
    """
    Attributes:
        project_id (int): The ID of the project to use.
        training_data (List['FineTuningSample']): The training file.
        model (Union[Unset, str]): ID of the model to use. See the model endpoint compatibility table for details.
        validaton_data (Union[Unset, List['FineTuningSample']]): The training file.
        num_epochs (Union[Unset, int]): The number of epochs to train for. Default: 1.
    """

    project_id: int
    training_data: List["FineTuningSample"]
    model: Union[Unset, str] = UNSET
    validaton_data: Union[Unset, List["FineTuningSample"]] = UNSET
    num_epochs: Union[Unset, int] = 1

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        project_id = self.project_id

        training_data = []
        for training_data_item_data in self.training_data:
            training_data_item = training_data_item_data.to_dict()
            training_data.append(training_data_item)

        model = self.model

        validaton_data: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.validaton_data, Unset):
            validaton_data = []
            for validaton_data_item_data in self.validaton_data:
                validaton_data_item = validaton_data_item_data.to_dict()
                validaton_data.append(validaton_data_item)

        num_epochs = self.num_epochs

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "project_id": project_id,
                "training_data": training_data,
            }
        )
        if model is not UNSET:
            field_dict["model"] = model
        if validaton_data is not UNSET:
            field_dict["validaton_data"] = validaton_data
        if num_epochs is not UNSET:
            field_dict["num_epochs"] = num_epochs

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        project_id = (
            self.project_id
            if isinstance(self.project_id, Unset)
            else (None, str(self.project_id).encode(), "text/plain")
        )

        _temp_training_data = []
        for training_data_item_data in self.training_data:
            training_data_item = training_data_item_data.to_dict()
            _temp_training_data.append(training_data_item)
        training_data = (None, json.dumps(_temp_training_data).encode(), "application/json")

        model = self.model if isinstance(self.model, Unset) else (None, str(self.model).encode(), "text/plain")

        validaton_data: Union[Unset, Tuple[None, bytes, str]] = UNSET
        if not isinstance(self.validaton_data, Unset):
            _temp_validaton_data = []
            for validaton_data_item_data in self.validaton_data:
                validaton_data_item = validaton_data_item_data.to_dict()
                _temp_validaton_data.append(validaton_data_item)
            validaton_data = (None, json.dumps(_temp_validaton_data).encode(), "application/json")

        num_epochs = (
            self.num_epochs
            if isinstance(self.num_epochs, Unset)
            else (None, str(self.num_epochs).encode(), "text/plain")
        )

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {key: (None, str(value).encode(), "text/plain") for key, value in self.additional_properties.items()}
        )
        field_dict.update(
            {
                "project_id": project_id,
                "training_data": training_data,
            }
        )
        if model is not UNSET:
            field_dict["model"] = model
        if validaton_data is not UNSET:
            field_dict["validaton_data"] = validaton_data
        if num_epochs is not UNSET:
            field_dict["num_epochs"] = num_epochs

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.fine_tuning_sample import FineTuningSample

        d = src_dict.copy() if src_dict else {}
        project_id = d.pop("project_id")

        training_data = []
        _training_data = d.pop("training_data")
        for training_data_item_data in _training_data:
            training_data_item = FineTuningSample.from_dict(training_data_item_data)

            training_data.append(training_data_item)

        model = d.pop("model", UNSET)

        validaton_data = []
        _validaton_data = d.pop("validaton_data", UNSET)
        for validaton_data_item_data in _validaton_data or []:
            validaton_data_item = FineTuningSample.from_dict(validaton_data_item_data)

            validaton_data.append(validaton_data_item)

        num_epochs = d.pop("num_epochs", UNSET)

        fine_tuning_input = cls(
            project_id=project_id,
            training_data=training_data,
            model=model,
            validaton_data=validaton_data,
            num_epochs=num_epochs,
        )

        fine_tuning_input.additional_properties = d
        return fine_tuning_input

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class DocumentOutputStatusEnum(str, Enum):
    CHUNKING = "CHUNKING"
    COMPLETED = "COMPLETED"
    FAILED = "FAILED"
    PARSING = "PARSING"
    PENDING = "PENDING"
    PROCESSING = "PROCESSING"
    UPLOADED = "UPLOADED"
    WAITING_FOR_CHUNKS_COMPLETION = "WAITING_FOR_CHUNKS_COMPLETION"

    def __str__(self) -> str:
        return str(self.value)



from typing import Dict, List, Type

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, TypedDict, TypeVar

from ..models.data_point_retrieve_gym_admin import DataPointRetrieveGymAdmin
from ..models.data_set_fine_tuning_job_gym_admin import DataSetFineTuningJobGymAdmin
from ..models.data_set_fine_tuning_request import DataSetFineTuningRequest
from ..models.data_set_project import DataSetProject

T = TypeVar("T", bound="DataSetRetrieveGymAdmin")


class DataSetRetrieveGymAdminDict(TypedDict):
    id: int
    project: "DataSetProject"
    datapoints: List["DataPointRetrieveGymAdmin"]
    training_finetuning_requests: List["DataSetFineTuningRequest"]
    training_finetuningjobs: List["DataSetFineTuningJobGymAdmin"]
    pass


@_attrs_define
class DataSetRetrieveGymAdmin:
    """
    Attributes:
        id (int):
        project (DataSetProject):
        datapoints (List['DataPointRetrieveGymAdmin']):
        training_finetuning_requests (List['DataSetFineTuningRequest']):
        training_finetuningjobs (List['DataSetFineTuningJobGymAdmin']):
    """

    id: int
    project: "DataSetProject"
    datapoints: List["DataPointRetrieveGymAdmin"]
    training_finetuning_requests: List["DataSetFineTuningRequest"]
    training_finetuningjobs: List["DataSetFineTuningJobGymAdmin"]

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        project = self.project.to_dict()

        datapoints = []
        for datapoints_item_data in self.datapoints:
            datapoints_item = datapoints_item_data.to_dict()
            datapoints.append(datapoints_item)

        training_finetuning_requests = []
        for training_finetuning_requests_item_data in self.training_finetuning_requests:
            training_finetuning_requests_item = training_finetuning_requests_item_data.to_dict()
            training_finetuning_requests.append(training_finetuning_requests_item)

        training_finetuningjobs = []
        for training_finetuningjobs_item_data in self.training_finetuningjobs:
            training_finetuningjobs_item = training_finetuningjobs_item_data.to_dict()
            training_finetuningjobs.append(training_finetuningjobs_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "project": project,
                "datapoints": datapoints,
                "training_finetuning_requests": training_finetuning_requests,
                "training_finetuningjobs": training_finetuningjobs,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.data_point_retrieve_gym_admin import DataPointRetrieveGymAdmin
        from ..models.data_set_fine_tuning_job_gym_admin import DataSetFineTuningJobGymAdmin
        from ..models.data_set_fine_tuning_request import DataSetFineTuningRequest
        from ..models.data_set_project import DataSetProject

        d = src_dict.copy() if src_dict else {}
        id = d.pop("id")

        project = DataSetProject.from_dict(d.pop("project"))

        datapoints = []
        _datapoints = d.pop("datapoints")
        for datapoints_item_data in _datapoints:
            datapoints_item = DataPointRetrieveGymAdmin.from_dict(datapoints_item_data)

            datapoints.append(datapoints_item)

        training_finetuning_requests = []
        _training_finetuning_requests = d.pop("training_finetuning_requests")
        for training_finetuning_requests_item_data in _training_finetuning_requests:
            training_finetuning_requests_item = DataSetFineTuningRequest.from_dict(
                training_finetuning_requests_item_data
            )

            training_finetuning_requests.append(training_finetuning_requests_item)

        training_finetuningjobs = []
        _training_finetuningjobs = d.pop("training_finetuningjobs")
        for training_finetuningjobs_item_data in _training_finetuningjobs:
            training_finetuningjobs_item = DataSetFineTuningJobGymAdmin.from_dict(training_finetuningjobs_item_data)

            training_finetuningjobs.append(training_finetuningjobs_item)

        data_set_retrieve_gym_admin = cls(
            id=id,
            project=project,
            datapoints=datapoints,
            training_finetuning_requests=training_finetuning_requests,
            training_finetuningjobs=training_finetuningjobs,
        )

        data_set_retrieve_gym_admin.additional_properties = d
        return data_set_retrieve_gym_admin

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from .client import AuthenticatedClient

from .api import (
    FinetuningAdminModule,
    DatapointsModule,
    EmbeddingsModule,
    FinetuningModule,
    ModelsModule,
    ChatModuleWrapper,
    RepositoryModuleWrapper,
)

class Prem:
    finetuning-admin: FinetuningAdminModule
    datapoints: DatapointsModule
    embeddings: EmbeddingsModule
    finetuning: FinetuningModule
    models: ModelsModule
    chat: ChatModuleWrapper
    repository: RepositoryModuleWrapper

    def __init__(self, api_key: str, base_url='https://app.premai.io'):
        client = AuthenticatedClient(token=api_key, base_url=base_url)
        # Init modules
        self.finetuning-admin = FinetuningAdminModule(client)
        self.datapoints = DatapointsModule(client)
        self.embeddings = EmbeddingsModule(client)
        self.finetuning = FinetuningModule(client)
        self.models = ModelsModule(client)
        self.chat = ChatModuleWrapper(client)
        self.repository = RepositoryModuleWrapper(client)

__all__ = [
    "Prem"
]


from enum import Enum


class StatusD09Enum(str, Enum):
    DONE = "done"
    FAILED = "failed"
    FINETUNING = "finetuning"
    QUEUED = "queued"

    def __str__(self) -> str:
        return str(self.value)



from typing import Dict, List, Type, Union

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, NotRequired, TypedDict, TypeVar

from ..models.status_d09_enum import StatusD09Enum
from ..types import UNSET, Unset

T = TypeVar("T", bound="DataSetFineTuningRequest")


class DataSetFineTuningRequestDict(TypedDict):
    id: int
    status: NotRequired[Union[Unset, StatusD09Enum]]
    pass


@_attrs_define
class DataSetFineTuningRequest:
    """
    Attributes:
        id (int):
        status (Union[Unset, StatusD09Enum]): * `queued` - Queued
            * `finetuning` - Fine tuning
            * `done` - Done
            * `failed` - Failed
    """

    id: int
    status: Union[Unset, StatusD09Enum] = UNSET

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        status: Union[Unset, str] = UNSET
        if not isinstance(self.status, Unset):
            status = self.status.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
            }
        )
        if status is not UNSET:
            field_dict["status"] = status

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy() if src_dict else {}
        id = d.pop("id")

        _status = d.pop("status", UNSET)
        status: Union[Unset, StatusD09Enum]
        if isinstance(_status, Unset):
            status = UNSET
        else:
            status = StatusD09Enum(_status)

        data_set_fine_tuning_request = cls(
            id=id,
            status=status,
        )

        data_set_fine_tuning_request.additional_properties = d
        return data_set_fine_tuning_request

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import json
from typing import Dict, List, Type

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from typing_extensions import Any, TypedDict, TypeVar

from ..models.data_point_create_gym_admin import DataPointCreateGymAdmin
from ..types import Unset

T = TypeVar("T", bound="DataSetCreateGymAdmin")


class DataSetCreateGymAdminDict(TypedDict):
    id: int
    project: int
    datapoints: List["DataPointCreateGymAdmin"]
    pass


@_attrs_define
class DataSetCreateGymAdmin:
    """
    Attributes:
        id (int):
        project (int):
        datapoints (List['DataPointCreateGymAdmin']):
    """

    id: int
    project: int
    datapoints: List["DataPointCreateGymAdmin"]

    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id

        project = self.project

        datapoints = []
        for datapoints_item_data in self.datapoints:
            datapoints_item = datapoints_item_data.to_dict()
            datapoints.append(datapoints_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "project": project,
                "datapoints": datapoints,
            }
        )

        return field_dict

    def to_multipart(self) -> Dict[str, Any]:
        id = self.id if isinstance(self.id, Unset) else (None, str(self.id).encode(), "text/plain")

        project = self.project if isinstance(self.project, Unset) else (None, str(self.project).encode(), "text/plain")

        _temp_datapoints = []
        for datapoints_item_data in self.datapoints:
            datapoints_item = datapoints_item_data.to_dict()
            _temp_datapoints.append(datapoints_item)
        datapoints = (None, json.dumps(_temp_datapoints).encode(), "application/json")

        field_dict: Dict[str, Any] = {}
        field_dict.update(
            {key: (None, str(value).encode(), "text/plain") for key, value in self.additional_properties.items()}
        )
        field_dict.update(
            {
                "id": id,
                "project": project,
                "datapoints": datapoints,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.data_point_create_gym_admin import DataPointCreateGymAdmin

        d = src_dict.copy() if src_dict else {}
        id = d.pop("id")

        project = d.pop("project")

        datapoints = []
        _datapoints = d.pop("datapoints")
        for datapoints_item_data in _datapoints:
            datapoints_item = DataPointCreateGymAdmin.from_dict(datapoints_item_data)

            datapoints.append(datapoints_item)

        data_set_create_gym_admin = cls(
            id=id,
            project=project,
            datapoints=datapoints,
        )

        data_set_create_gym_admin.additional_properties = d
        return data_set_create_gym_admin

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
